/* $Id: layout.h,v 1.1.1.1 2002/02/08 00:23:42 mquinson Exp $ */

/* layout - compute where to put each machines in the visualization space    */
/* Author: Martin Quinson                                                    */
/* License: LGPL                                                             */

#ifndef LAYOUT_H
#define LAYOUT_H
/*====[ Layout functions ]===================================================*/
/* The algorithm used is the folowing:                                       */
/*                                                                           */
/* Phase 1: Count in how many clique are each nodes (memberof)               */
/*                                                                           */
/* Phase 2: Figure out cliques and machines level                            */
/*   Step 1: level of each clique = average memberof of its members          */
/*   Step 2: Renumerate the level to be a sorted suite of int                */
/*           (1.3 ; 3 ; 3.5-> 1 ; 2 ; 3)                                     */
/*   Step 3: merge adjacent levels with no common host                       */
/*   Step 4: We computed a valid level setup. Now, some cosmetic changes:    */
/*            move all cliques to the level upstairs from were it is having  */
/*            no comon host with the clique. We want to do this change:      */
/*                                                                           */
/*    level2:     (A B C D)             \           (A B C D)                */
/*    level1:     (A E F G)              }-> (A E F G) (B H I J) (C K L)     */
/*    level0: (B H I J) (C K L) (E M N) /            (E M N)                 */
/*     because cliques (BHIJ) and (CKL) are free to go to level 1            */
/*                                                                           */
/*     (top-bottom algorithm)                                                */
/*                                                                           */
/* Phase 3: Compute the position of hosts (bottom-top-bottom algorithm)      */
/*   bottom top: set clique member pos relative to the top of the clique.    */
/*     Placement is done on a circle (cone-tree algo).                       */
/*     Each node are allocated some room around this circle:                 */
/*        leaf take 1, roots take (sum of subtree).                          */
/*   top bottom: root is {0,0,0}, so all absolute position of each node      */
/*     may be computed                                                       */
/*   Step 3: normalize positions (>0 and fitted to display space)            */
/*                                                                           */
/* Here we are, we can display this stuff.                                   */
/*                                                                           */
/*                                                                           */
/* ISSUES (TODO):                                                            */
/*   - We suppose the set of machines to be connexe. What if a machine runs  */
/*     no net experiment, or if there is several non-connected sets ?        */
/*   - Is phase 3, while placing the nodes on the circle, we should take     */
/*     care of links between nodes there and nodes in another subtree.       */
/*     Ie, if we have a|(bc)(de) (a on top, two sub trees bc and de),        */
/*     plus a connexion between b and d, we should make sure b goes on the   */
/*     the nearest point from d on its circle.                               */
/*     Ie, doing:     a       and not     a                                  */
/*                (cb) (de)           (bc) (de)                              */
/*       so that the links (bc) and (bd) do not overlap on visualization     */
/*                                                                           */
/*     I have no fucking idea on how to implement that, though.              */
/*===========================================================================*/

/*----[ layout_compute ]-----------------------------------------------------*/
/* Call all phases in order. Kinda main() for layout.                        */
/*---------------------------------------------------------------------------*/
int layout_compute();

#endif /* LAYOUT_H */
