/* $Id: machines_cliques.h,v 1.1.1.1 2002/02/08 00:23:44 mquinson Exp $ */

/* machines_cliques - tool functions to handle sets of machines and cliques  */
/* Author: Martin Quinson                                                    */
/* License: LGPL                                                             */

#ifndef MACHINES_CLIQUES_H
#define MACHINES_CLIQUES_H

#define DEFAULT_NAME_SERVER_PORT "8090"
#define DEFAULT_SENSOR_PORT      "8060"


typedef struct machine_ machine_t;
struct machine_ {
  char fqdn[80];
  char sname[80]; /* short name */
  int port;
  char series[128];

  /* tmp stuff to compute the layout */
  int memberof,weight; 
  machine_t *father;   

  double x,y,z;
  int number; /* Index of this in the table used for rendering */
};

typedef struct {
  char name[80];
  int count;
  machine_t **machine;
  int level;
} clique_t;

extern JRB machines;
extern JRB cliques;

/*====[ Machines stuff ]=====================================================*/

/*----[ machine_new ]--------------------------------------------------------*/
/* Creates a machine called #name# (If not hostname:port format, 8060 is     */
/*   assumed), and insert it to the global list of machines.                 */
/*---------------------------------------------------------------------------*/
int machine_new(const char *name);
/*----[ machines_free ]------------------------------------------------------*/
/* Memory leak is a Bad Thing.  (frees the whole set)                        */
/*---------------------------------------------------------------------------*/
void machines_free();
/*----[ machines_search ]----------------------------------------------------*/
/* search all machines runing a sensor from the nameserver.                  */
/*---------------------------------------------------------------------------*/
int machines_search();
/*----[ machines_dump ]------------------------------------------------------*/
/* Dumps all machines for debugging purpose                                  */
/*---------------------------------------------------------------------------*/
int machines_dump(JRB set);
/*----[ machines_display ]---------------------------------------------------*/
/* Makes a textual display of the positions:                                 */
/*   Projection on (Ox,Oz) to see z                                          */
/*   Projection on (Ox,Oy) to see x and y                                    */
/*---------------------------------------------------------------------------*/
int machines_display();

/*====[ Clique functions ]===================================================*/
/*----[ clique_new ]---------------------------------------------------------*/
/* Creates a clique named #name#, containing #members# (comma separated list)*/
/*  on #layer#, and insert it to the global list of cliques.                 */
/*---------------------------------------------------------------------------*/
int clique_new(char *name,char *members,int layer);
/*----[ cliques_free ]-------------------------------------------------------*/
/* Memory leak is a Bad Thing.  (frees the whole set)                        */
/*---------------------------------------------------------------------------*/
void cliques_free();
/*----[ cliques_search ]-----------------------------------------------------*/
/* search all cliques runing a sensor from the nameserver.                   */
/*---------------------------------------------------------------------------*/
int cliques_search();
/*----[ cliques_dump ]-------------------------------------------------------*/
/* Dumps all cliques for debugging purpose                                   */
/*---------------------------------------------------------------------------*/
int cliques_dump(JRB set);

/*----[ data_fake ]----------------------------------------------------------*/
/* Fakes data with #nbcliques#, each one been described with a subsequent    */
/*  string (debuging purpose. Strings are coma (,) separated:                */
/*    The first word there is the name of the clique.                        */
/*    The next ones are name of machines in the clique.                      */
/* This is caller responsability to make sure that there is enough string for*/
/*  #nbcliques#.                                                             */
/*---------------------------------------------------------------------------*/
int data_fake(int nbcliques, ...);
#endif /* MACHINES_CLIQUES_H */
